@@ -16,39 +16,47 @@ from gremlinapi.exceptions import (
 from gremlinapi.config import GremlinAPIConfig
 from gremlinapi.util import get_version
 
-try:
-    import requests
-    import requests.adapters
-except ImportError:
-    requests = None
-    import urllib3
+from typing import Tuple, Union, Optional, Any, Dict, Callable, Type
 
-log = logging.getLogger("GremlinAPI.client")
+import requests
+import urllib3  # type: ignore
+
+# try:
+#     import requests
+#     import requests.adapters
+# except ImportError:
+#     del requests
+#     import urllib3 # type: ignore
+
+log: logging.Logger = logging.getLogger("GremlinAPI.client")
 
 
 class GremlinAPIHttpClient(object):
     @classmethod
-    def api_call(cls, method, uri, *args, **kwargs):
-        error_message = f"This function is not implemented, please consume the proper http library for your environment"
-        log.fatal(error_message)
-        raise NotImplementedError(error_message)
+    def api_call(
+        cls, method: str, endpoint: str, *args: tuple, **kwargs: dict
+    ) -> Tuple[Union[requests.Response, urllib3.HTTPResponse], dict]:
+        if requests:
+            return GremlinAPIRequestsClient.api_call(method, endpoint, *args, **kwargs)
+        else:
+            return GremlinAPIurllibClient.api_call(method, endpoint, *args, **kwargs)
 
     @classmethod
-    def base_uri(cls, uri):
-        if not uri.startswith("http") and GremlinAPIConfig.base_uri not in uri:
+    def base_uri(cls, uri: str) -> str:
+        if not uri.startswith("http") and str(GremlinAPIConfig.base_uri) not in uri:
             uri = f"{GremlinAPIConfig.base_uri}{uri}"
         return uri
 
     @classmethod
-    def header(cls, *args, **kwargs):
-        api_key = kwargs.get("api_key", None)
-        bearer_token = kwargs.get("bearer_token", None)
-        header = dict()
+    def header(cls, *args: tuple, **kwargs: dict) -> dict:
+        api_key: Union[dict, str] = kwargs.get("api_key", "")
+        bearer_token: Union[dict, str] = kwargs.get("bearer_token", "")
+        header: dict = dict()
         if not (api_key and bearer_token):
             if GremlinAPIConfig.bearer_token:
-                bearer_token = GremlinAPIConfig.bearer_token
+                bearer_token = str(GremlinAPIConfig.bearer_token)
             if GremlinAPIConfig.api_key:
-                api_key = GremlinAPIConfig.api_key
+                api_key = str(GremlinAPIConfig.api_key)
         if api_key and not bearer_token:
             if "Key" in api_key:
                 header["Authorization"] = api_key
@@ -60,35 +68,39 @@ class GremlinAPIHttpClient(object):
             else:
                 header["Authorization"] = f"Bearer {bearer_token}"
         else:
-            error_msg = f"Missing API Key or Bearer Token, none supplied: {api_key}, {bearer_token}"
-            log.fatal(error_msg)
+            error_msg: str = f"Missing API Key or Bearer Token, none supplied: {api_key}, {bearer_token}"
+            log.error(error_msg)
             # raise HTTPBadHeader(error_msg)
         header["X-Gremlin-Agent"] = f"gremlin-sdk-python/{get_version()}"
         return header
 
     @classmethod
-    def proxies(cls):
-        error_message = f"This function is not implemented, please consume the proper http library for your environment"
-        log.fatal(error_message)
-        raise NotImplementedError(error_message)
+    def proxies(cls) -> dict:
+        if requests:
+            return GremlinAPIRequestsClient.proxies()
+        else:
+            error_message: str = (
+                f"This function is not implemented, proxiea not supported for urllib"
+            )
+            log.error(error_message)
+            raise NotImplementedError(error_message)
 
 
-class GremlineAPIRequestsClient(GremlinAPIHttpClient):
+class GremlinAPIRequestsClient(GremlinAPIHttpClient):
     @classmethod
-    def proxies(cls):
-        proxies = dict()
-
+    def proxies(cls) -> dict:
+        proxies: dict = dict()
         if GremlinAPIConfig.http_proxy and type(GremlinAPIConfig.http_proxy) is str:
             proxies["http"] = GremlinAPIConfig.http_proxy
         if GremlinAPIConfig.https_proxy and type(GremlinAPIConfig.https_proxy) is str:
             proxies["https"] = GremlinAPIConfig.https_proxy
-
         return proxies
 
     @classmethod
-    def api_call(cls, method, endpoint, *args, **kwargs):
-
-        request_methods = {
+    def api_call(
+        cls, method: str, endpoint: str, *args: tuple, **kwargs: dict
+    ) -> Tuple[requests.Response, dict]:
+        request_methods: Dict[str, Callable] = {
             "HEAD": requests.head,
             "GET": requests.get,
             "POST": requests.post,
@@ -96,12 +108,10 @@ class GremlineAPIRequestsClient(GremlinAPIHttpClient):
             "DELETE": requests.delete,
             "PATCH": requests.patch,
         }
-
-        # uri = f'{GremlinAPIConfig.base_uri}{endpoint}'
-        uri = cls.base_uri(endpoint)
-        client = request_methods.get(method.upper())
-        raw_content = kwargs.pop("raw_content", False)
-        data = None
+        uri: str = cls.base_uri(endpoint)
+        client: Union[Callable, Any] = request_methods.get(method.upper())
+        raw_content: dict = kwargs.pop("raw_content", {})
+        data: Union[dict, str] = {}
         if "data" in kwargs:
             data = kwargs.pop("data")
         elif "body" in kwargs:
@@ -118,17 +128,19 @@ class GremlineAPIRequestsClient(GremlinAPIHttpClient):
             log.debug(f"httpd client kwargs: {kwargs}")
 
         if data:
-            resp = client(uri, data=data, allow_redirects=False, **kwargs)
+            resp: requests.Response = client(
+                uri, data=data, allow_redirects=False, **kwargs
+            )
         else:
             resp = client(uri, allow_redirects=False, **kwargs)
 
         if resp.status_code >= 400:
-            error_msg = f"error {resp.status_code} : {resp.reason}"
+            error_msg: str = f"error {resp.status_code} : {resp.reason}"
             log.warning(error_msg)
             if log.getEffectiveLevel() == logging.DEBUG:
-                log.debug(f"{client}\n{uri}\n{data}\n{kwargs}")
+                log.debug(f"{uri}\n{data}\n{kwargs}")
             raise HTTPError(error_msg)
-
+        body: Any = None
         if raw_content:
             body = resp.content
         else:
@@ -148,37 +160,44 @@ class GremlineAPIRequestsClient(GremlinAPIHttpClient):
 class GremlinAPIurllibClient(GremlinAPIHttpClient):
     """Fallback library in the event requests library is unavailable."""
 
+    import urllib3
+
     @classmethod
-    def api_call(cls, method, endpoint, *args, **kwargs):
+    def api_call(
+        cls, method: str, endpoint: str, *args: tuple, **kwargs: dict
+    ) -> Tuple[urllib3.HTTPResponse, dict]:
 
         log.warning(
             f"The request to {endpoint} is using the urllib3 library. Consider installing th requests library."
         )
-        form_data = None
-        request_body = None
 
         if "data" in kwargs:
-            form_data = kwargs.pop("data")
+            form_data: dict = kwargs.pop("data")
         elif "body" in kwargs:
             if "Content-Type" not in kwargs["headers"]:
                 kwargs["headers"]["Content-Type"] = "application/json"
-            request_body = json.dumps(kwargs.pop("body"))
+            request_body: str = json.dumps(kwargs.pop("body"))
 
-        uri = f"{GremlinAPIConfig.base_uri}{endpoint}"
+        uri: str = f"{GremlinAPIConfig.base_uri}{endpoint}"
 
         if GremlinAPIConfig.https_proxy and type(GremlinAPIConfig.https_proxy) is str:
-            http_client = urllib3.ProxyManager(GremlinAPIConfig.https_proxy)
+            http_client: urllib3.ProxyManager = urllib3.ProxyManager(
+                GremlinAPIConfig.https_proxy
+            )
         elif GremlinAPIConfig.http_proxy and type(GremlinAPIConfig.http_proxy) is str:
             http_client = urllib3.ProxyManager(GremlinAPIConfig.http_proxy)
         else:
             http_client = urllib3.PoolManager()
 
         if form_data:
-            resp = http_client.request(method, uri, fields=form_data, **kwargs)
+            resp: urllib3.HTTPResponse = http_client.request(
+                method, uri, fields=form_data, **kwargs
+            )
         elif request_body:
             resp = http_client.request(method, uri, body=request_body, **kwargs)
         else:
             resp = http_client.request(method, uri, **kwargs)
+
         if log.getEffectiveLevel() == logging.DEBUG:
             log.debug(resp)
 
@@ -189,12 +208,9 @@ class GremlinAPIurllibClient(GremlinAPIHttpClient):
                 )
             raise HTTPError(resp)
 
-        body = json.loads(resp.data)
+        body: dict = json.loads(resp.data)
         return resp, body
 
 
-def get_gremlin_httpclient():
-    if requests:
-        return GremlineAPIRequestsClient
-    else:
-        return GremlinAPIurllibClient
+def get_gremlin_httpclient() -> Type[GremlinAPIHttpClient]:
+    return GremlinAPIHttpClient
